@@ -1,7 +1,7 @@
 /* hash.h - Manage hashes for cached dns records
-   Copyright (C) 2000 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2003.
+   Copyright (C) 2000 Thomas Moestl
+   Copyright (C) 2003 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -33,17 +33,25 @@ typedef struct dns_hash_ent_s {
 	dns_cent_t		*data;
 } dns_hash_ent_t;
 
-/* Redefine this if you want another hash size. Should work ;-). The number of hash buckets is computed as power of two;
- * so, eg HASH_SZ set to 10 yields 1024 hash rows (2^10 or 1<<10). Only powers of two are possible conveniently. 
- * If you modify HASH_NUM_BUCKETS, also change HASH_SZ! HASH_SZ may not be bigger than 32 (if you set it even close to
- * that value, you are nuts.)*/ 
-#define HASH_NUM_BUCKETS 1024
-#define HASH_SZ            10
+/* Redefine this if you want another hash size. Should work ;-).
+ * The number of hash buckets is computed as power of two;
+ * so, e.g. HASH_SZ set to 10 yields 1024 hash rows (2^10 or 1<<10).
+ * Only powers of two are possible conveniently. 
+ * HASH_SZ may not be bigger than 32 (if you set it even close to that value,
+ * you are nuts.) */ 
+/* #define HASH_SZ       10 */  /* Now defined in config.h */
+#define HASH_NUM_BUCKETS (1<<HASH_SZ)
 
 #define HASH_BITMASK     (HASH_NUM_BUCKETS-1)
 
 extern dns_hash_ent_t *hash_buckets[];
 
+/* A type for remembering the position in the hash table where a new entry can be inserted. */
+typedef struct {
+	dns_hash_ent_t **pos;      /* pointer to the location in the hash table */
+	unsigned long  rhash;      /* long hash */
+} dns_hash_loc_t;
+
 /* A type for position specification for fetch_first and fetch_next */
 typedef struct {
 	int            bucket;     /* bucket chain we are in */
@@ -57,15 +65,16 @@ inline static void mk_dns_hash()
 		hash_buckets[i]=NULL;
 }
 
-void add_dns_hash(dns_cent_t *data);
+dns_cent_t *dns_lookup(const unsigned char *key, dns_hash_loc_t *loc);
+void add_dns_hash(dns_cent_t *data, dns_hash_loc_t *loc);
+dns_cent_t *del_dns_hash_ent(dns_hash_loc_t *loc);
 dns_cent_t *del_dns_hash(const unsigned char *key);
-dns_cent_t *dns_lookup(const unsigned char *key);
 void free_dns_hash();
 
 dns_cent_t *fetch_first(dns_hash_pos_t *pos);
 dns_cent_t *fetch_next(dns_hash_pos_t *pos);
 
-#ifdef DBGHASH
+#ifdef DEBUG_HASH
 void dumphash();
 #endif
 
