@@ -1,7 +1,7 @@
 /* icmp.c - Server response tests using ICMP echo requests
-   Copyright (C) 2000, 2001 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2003.
+   Copyright (C) 2000, 2001 Thomas Moestl
+   Copyright (C) 2003 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -21,13 +21,15 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 /*
- * This should now work on both Linux and FreeBSD. If anyone
+ * This should now work on both Linux and FreeBSD (and CYGWIN?). If anyone
  * with experience in other Unix flavors wants to contribute platform-specific
  * code, he is very welcome. 
  */
 
 #include <config.h>
+#ifdef HAVE_SYS_POLL_H
 #include <sys/poll.h>
+#endif
 #include <sys/time.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -36,15 +38,21 @@ Boston, MA 02111-1307, USA.  */
 #include <errno.h>
 #include <string.h>
 #include "ipvers.h"
-#if TARGET==TARGET_BSD
-# include <netinet/in_systm.h>
-#endif
-#include <netinet/ip.h>
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX)
+# include <netinet/ip.h>
 # include <linux/types.h>
 # include <linux/icmp.h>
-#else
+#elif (TARGET==TARGET_BSD)
+# include <netinet/in_systm.h>
+# include <netinet/ip.h>
 # include <netinet/ip_icmp.h>
+#elif (TARGET==TARGET_CYGWIN)
+# include <netinet/ip.h>
+# include <netinet/in_systm.h>
+# include <netinet/ip_icmp.h>
+# include "freebsd_netinet_ip_icmp.h"
+#else
+# error Unsupported platform!
 #endif
 #ifdef ENABLE_IPV6
 # include <netinet/ip6.h>
@@ -59,7 +67,7 @@ Boston, MA 02111-1307, USA.  */
 static char rcsid[]="$Id: icmp.c,v 1.26 2002/01/14 17:39:26 tmm Exp $";
 #endif
 
-#define ICMP_MAX_ERRS 5
+#define ICMP_MAX_ERRS 10
 volatile unsigned long icmp_errs=0; /* This is only here to minimize log output. Since the 
 				       consequences of a race is only one log message more/less
 				       (out of ICMP_MAX_ERRS), no lock is required. */
@@ -70,35 +78,33 @@ volatile int ping6_isocket=-1;
 #endif
 
 /* different names, same thing... be careful, as these are macros... */
-#if TARGET==TARGET_BSD
-# define icmphdr   icmp
-# define iphdr     ip
-# define ip_saddr  ip_src.s_addr
-# define ip_daddr  ip_dst.s_addr
-#else
+#if (TARGET==TARGET_LINUX)
 # define ip_saddr  saddr
 # define ip_daddr  daddr
 # define ip_hl     ihl
 # define ip_p	   protocol
+#else
+# define icmphdr   icmp
+# define iphdr     ip
+# define ip_saddr  ip_src.s_addr
+# define ip_daddr  ip_dst.s_addr
 #endif
 
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX)
 # define icmp_type  type
 # define icmp_code  code
 # define icmp_cksum checksum
 # define icmp_id un.echo.id
 # define icmp_seq un.echo.sequence
-#endif
-
-#if TARGET==TARGET_BSD
-# define ICMP_DEST_UNREACH   ICMP_UNREACH
+#else
+# define ICMP_DEST_UNREACH  ICMP_UNREACH
 # define ICMP_TIME_EXCEEDED ICMP_TIMXCEED    
 #endif
 
 #define ICMP_BASEHDR_LEN  8
 #define ICMP4_ECHO_LEN    ICMP_BASEHDR_LEN
 
-#if (TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD)
+#if (TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)
 /*
  * These are the ping implementations for Linux/FreeBSD in their IPv4/ICMPv4 and IPv6/ICMPv6 versions.
  * I know they share some code, but I'd rather keep them separated in some parts, as some
@@ -131,7 +137,7 @@ void init_ping_socket()
 #endif
 }
 
-/* Takes a packet as send out and a recieved ICMP packet and looks whether the ICMP packet is 
+/* Takes a packet as send out and a received ICMP packet and looks whether the ICMP packet is 
  * an error reply on the sent-out one. packet is only the packet (without IP header).
  * errmsg includes an IP header.
  * to is the destination address of the original packet (the only thing that is actually
@@ -165,11 +171,8 @@ static int ping4(struct in_addr addr, int timeout, int rep)
 {
 	int i;
 	int isock;
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX)
 	struct icmp_filter f;
-#else
-	struct protoent *pe;
-	int SOL_IP;
 #endif
 	struct sockaddr_in from,to;
 	struct icmphdr icmpd;
@@ -177,17 +180,9 @@ static int ping4(struct in_addr addr, int timeout, int rep)
 	unsigned short *ptr;
 	unsigned short id=(unsigned short)get_rand16(); /* randomize a ping id */
 
-#if TARGET!=TARGET_LINUX	
-	if (!(pe=getprotobyname("ip"))) {
-		log_warn("icmp ping: getprotobyname() failed: %s",strerror(errno));
-		return -1;
-	}
-	SOL_IP=pe->p_proto;
-#endif
-
 	isock=ping_isocket;
 
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX)
 	/* Fancy ICMP filering -- only on Linux (as far is I know) */
 	
 	/* In fact, there should be macros for treating icmp_filter, but I haven't found them in Linux 2.2.15.
@@ -265,10 +260,11 @@ static int ping4(struct in_addr addr, int timeout, int rep)
 				break;
 
 #ifdef NO_POLL
-			if (FD_ISSET(isock,&fds) || FD_ISSET(isock,&fdse)) {
+			if (FD_ISSET(isock,&fds) || FD_ISSET(isock,&fdse))
 #else
-			if (pfd.revents&POLLIN || pfd.revents&POLLERR) {
+			if (pfd.revents&(POLLIN|POLLERR))
 #endif
+			{
 				char buf[1024];
 				socklen_t sl=sizeof(from);
 				int len;
@@ -298,6 +294,12 @@ static int ping4(struct in_addr addr, int timeout, int rep)
 					return -1; /* error */
 				}
 			}
+			else {
+				if (++icmp_errs<=ICMP_MAX_ERRS) {
+					log_error("Unhandled poll/select event in ping4() at %s, line %d.",__FILE__,__LINE__);
+				}
+				return -1; 
+			}
 			tpassed=time(NULL)-tm;
 		} while (tpassed<timeout);
 	}
@@ -307,7 +309,7 @@ static int ping4(struct in_addr addr, int timeout, int rep)
 
 #ifdef ENABLE_IPV6
 
-/* Takes a packet as send out and a recieved ICMPv6 packet and looks whether the ICMPv6 packet is 
+/* Takes a packet as send out and a received ICMPv6 packet and looks whether the ICMPv6 packet is 
  * an error reply on the sent-out one. packet is only the packet (without IPv6 header).
  * errmsg does not include an IPv6 header. to is the address the sent packet went to.
  * This is specialized for icmpv6: It zeros out the checksum field, which is filled in
@@ -429,10 +431,11 @@ static int ping6(struct in6_addr a, int timeout, int rep)
 				break;
 
 #ifdef NO_POLL
-			if (FD_ISSET(isock,&fds) || FD_ISSET(isock,&fdse)) {
+			if (FD_ISSET(isock,&fds) || FD_ISSET(isock,&fdse))
 #else
-			if (pfd.revents&POLLIN || pfd.revents&POLLERR) {
+			if (pfd.revents&(POLLIN|POLLERR))
 #endif
+			{
 				char buf[1024];
 				socklen_t sl=sizeof(from);
 				int len;
@@ -457,6 +460,12 @@ static int ping6(struct in6_addr a, int timeout, int rep)
 					return -1; /* error */
 				}
 			}
+			else {
+				if (++icmp_errs<=ICMP_MAX_ERRS) {
+					log_error("Unhandled poll/select event in ping6() at %s, line %d.",__FILE__,__LINE__);
+				}
+				return -1; 
+			}
 			tpassed=time(NULL)-tm;
 		} while (tpassed<timeout);
 	}
@@ -501,4 +510,4 @@ int ping(pdnsd_a *addr, int timeout, int rep)
 
 #else
 # error "Huh! No OS macro defined!"
-#endif /*TARGET==TARGET_LINUX || TARGET==TARGET_BSD*/
+#endif /*(TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)*/
