@@ -1,7 +1,7 @@
 /* netdev.c - Test network devices for existence and status
-   Copyright (C) 2000, 2001 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2002, 2003.
+   Copyright (C) 2000, 2001 Thomas Moestl
+   Copyright (C) 2002, 2003 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -75,19 +75,25 @@ Boston, MA 02111-1307, USA.  */
 static char rcsid[]="$Id: netdev.c,v 1.11 2001/06/13 17:28:04 tmm Exp $";
 #endif
 
+#if (TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD)
 /* Taken from FreeBSD net/if.h rev. 1.58.2.1 */
 #define	SIZEOF_ADDR_IFREQ(ifr) \
 	((ifr).ifr_addr.sa_len > sizeof(struct sockaddr) ? \
 	 (sizeof(struct ifreq) - sizeof(struct sockaddr) + \
 	  (ifr).ifr_addr.sa_len) : sizeof(struct ifreq))
+#elif (TARGET==TARGET_CYGWIN)
+#define SIZEOF_ADDR_IFREQ(ifr) (sizeof(struct sockaddr))
+#else
+# error Unsupported platform!
+#endif
 
 /*
  * These portion is Linux/FreeBSD specific. Please write interface-detection routines for other
  * flavours of Unix if you can and want.
  */
 
-#if (TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD)
-# if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)
+# if (TARGET==TARGET_LINUX)
 
 volatile int isdn_errs=0;
 
@@ -168,7 +174,7 @@ int dev_up(char *ifname, char *devname)
 }
  
 
-# endif
+# endif /*(TARGET==TARGET_LINUX)*/
 
 /*
  * Test whether the network device specified in devname is up and
@@ -182,10 +188,9 @@ int dev_up(char *ifname, char *devname)
  */
 int if_up(char *devname)
 {
-        struct protoent *pe;
 	int sock;
 	struct ifreq ifr;
-# if TARGET==TARGET_LINUX
+# if (TARGET==TARGET_LINUX)
 	unsigned int devnamelen=strlen(devname);
 	if (devnamelen>4 && devnamelen<=6 && strncmp(devname,"ippp",4)==0) {
 		/* This function didn't manage the interface uptest correctly. Thanks to
@@ -204,9 +209,7 @@ int if_up(char *devname)
 		/* If it doesn't match our rules for isdn devices, treat as normal if */
 	}
 # endif
-	if (!(pe=getprotobyname("udp")))
-		return 0;
-	if ((sock=socket(PF_INET,SOCK_DGRAM, pe->p_proto))==-1)
+	if ((sock=socket(PF_INET,SOCK_DGRAM, IPPROTO_UDP))==-1)
 		return 0;
 	strncp(ifr.ifr_name,devname,IFNAMSIZ);
 	if (ioctl(sock,SIOCGIFFLAGS,&ifr)==-1) {
@@ -217,18 +220,15 @@ int if_up(char *devname)
 	return (ifr.ifr_flags&IFF_UP) && (ifr.ifr_flags&IFF_RUNNING);
 }
 
-# if TARGET==TARGET_LINUX
+# if (TARGET==TARGET_LINUX)
 
 int is_local_addr(pdnsd_a *a)
 {
 #  ifdef ENABLE_IPV4
 	if (run_ipv4) {
 		int res=0,i,sock;
-		struct protoent *pe;
 		struct ifreq ifr;
-		if (!(pe=getprotobyname("udp")))
-			return 0;
-		if ((sock=socket(PF_INET,SOCK_DGRAM, pe->p_proto))==-1)
+		if ((sock=socket(PF_INET,SOCK_DGRAM, IPPROTO_UDP))==-1)
 			return 0;
 		for (i=1;i<255;i++) {
 			ifr.ifr_ifindex=i;
@@ -293,11 +293,10 @@ int is_local_addr(pdnsd_a *a)
 	return 0;
 }
 
-# else
+# else /*(TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)*/
 
 int is_local_addr(pdnsd_a *a)
 {
-	struct protoent *pe;
 	int sock;
         struct ifconf ifc;
 	char buf[2048];
@@ -306,11 +305,9 @@ int is_local_addr(pdnsd_a *a)
 	char *ad;
 	  
 
-	ifc.ifc_len=2048;
+	ifc.ifc_len=sizeof(buf);
 	ifc.ifc_buf=buf;
-	if (!(pe=getprotobyname("udp")))
-		return 0;
-	if ((sock=socket(PF_INET,SOCK_DGRAM, pe->p_proto))==-1)
+	if ((sock=socket(PF_INET,SOCK_DGRAM, IPPROTO_UDP))==-1)
 		return 0;
 	if (ioctl(sock,SIOCGIFCONF,&ifc)==-1) {
 	        return 0;
