@@ -1,7 +1,7 @@
 /* pdnsd-ctl.c - Control pdnsd through a pipe
-   Copyright (C) 2000, 2001 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2002, 2003, 2004.
+   Copyright (C) 2000, 2001 Thomas Moestl
+   Copyright (C) 2002, 2003, 2004 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -23,6 +23,9 @@ Boston, MA 02111-1307, USA.  */
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -39,6 +42,11 @@ Boston, MA 02111-1307, USA.  */
 #include "../rr_types.h"
 #include "../cache.h"
 
+#if !defined(HAVE_ALLOCA) && !defined(alloca)
+#define alloca malloc
+#endif
+
+
 #if !defined(lint) && !defined(NO_RCSIDS)
 static char rcsid[]="$Id: pdnsd-ctl.c,v 1.19 2001/12/30 15:29:43 tmm Exp $";
 #endif
@@ -50,10 +58,17 @@ typedef struct {
 	int  val;
 } cmd_s;
 
-#define CTL_AUX_LISTRRT 16
+#define CMD_LIST_RRTYPES (CTL_MAX+1)
+#define CMD_HELP         (CTL_MAX+2)
+#define CMD_VERSION      (CTL_MAX+3)
 
-static const cmd_s top_cmds[]={{"status",CTL_STATS},{"server",CTL_SERVER},{"record",CTL_RECORD},
-		   {"source",CTL_SOURCE},{"add",CTL_ADD},{"neg",CTL_NEG}, {NULL,0}};
+static const cmd_s top_cmds[]={
+	{"help",CMD_HELP},{"version",CMD_VERSION},{"list-rrtypes",CMD_LIST_RRTYPES},
+	{"status",CTL_STATS},{"server",CTL_SERVER},{"record",CTL_RECORD},
+	{"source",CTL_SOURCE},{"add",CTL_ADD},{"neg",CTL_NEG},
+	{"config",CTL_CONFIG},{"empty-cache",CTL_EMPTY}, {"dump",CTL_DUMP},
+	{NULL,0}
+};
 static const cmd_s server_cmds[]= {{"up",CTL_S_UP},{"down",CTL_S_DOWN},{"retest",CTL_S_RETEST},{NULL,0}};
 static const cmd_s record_cmds[]= {{"delete",CTL_R_DELETE},{"invalidate",CTL_R_INVAL},{NULL,0}};
 static const cmd_s onoff_cmds[]= {{"off",0},{"on",1},{NULL,0}};
@@ -72,15 +87,15 @@ static const char *help_messages[] =
 
 	"Command line options\n"
 
-	"-c\tcachedir\n\tset the cache directory to cachedir (must match pdnsd setting)\n"
-	"-q\n\tbe quiet unless output is specified by command or something goes wrong\n"
+	"-c\tcachedir\n\tSet the cache directory to cachedir (must match pdnsd setting).\n"
+	"-q\n\tBe quiet unless output is specified by command or something goes wrong.\n"
 
 	"Commands and needed options are:\n"
 
-	"help\t[no options]\n\tprint this help\n"
-	"version\t[no options]\n\tprint version info\n",
+	"help\t[no options]\n\tPrint this help.\n"
+	"version\t[no options]\n\tPrint version info.\n",
 
-	"status\t[no options]\n\tprint pdnsd's status\n",
+	"status\t[no options]\n\tPrint pdnsd's status.\n",
 
 	"server\t(index|label)\t(up|down|retest)\t[dns1[,dns2[,...]]]\n"
 	"\tSet the status of the server with the given index to up or down, or\n"
@@ -128,13 +143,29 @@ static const char *help_messages[] =
  	"\tbefore adding records.\n",
 
 	"neg\tname\t[type]\t[ttl]\n"
- 	"\tAdd a negative cached record to pdnsd's cache, replacing existing\n"
+ 	"\tAdd a negatively cached record to pdnsd's cache, replacing existing\n"
 	"\trecords for the same name and type. If no type is given, the whole\n"
-	"\tdomain is cached negative. For negative cached records, errors are\n"
+	"\tdomain is cached negatively. For negatively cached records, errors are\n"
 	"\timmediately returned on a query, without querying other servers first.\n"
 	"\tThe ttl is optional, the default is 900 seconds.\n",
 
-	"list-rrtypes\n"
+	"config\t[filename]\n"
+	"\tReload pdnsd's configuration file.\n"
+	"\tThe config file must be owned by the uid that pdnsd had when it was\n"
+	"\tstarted, and be readable by pdnsd's run_as uid. If no file name is\n"
+	"\tspecified, the config file used at start up is reloaded.\n",
+
+	"empty-cache\t[no options]\n"
+	"\tEmpty the cache completely, freeing all existing entries.\n"
+	"\tThis also removes \"local\" records, as defined by the config file.\n"
+	"\tTo restore local records, run \"pdnsd-ctl config\" immediately afterwards.\n",
+
+	"dump\t[name]\n"
+	"\tPrint information stored in the cache about name.\n"
+	"\tIf name is missing, information about all the names in the cache will\n"
+	"\tbe printed.\n",
+
+	"list-rrtypes\t[no options]\n"
 	"\tList available rr types for the neg command. Note that those are only\n"
 	"\tused for the neg command, not for add!\n"
 };
@@ -144,7 +175,7 @@ static const char *help_messages[] =
 static int open_sock(const char *cache_dir)
 {
 	struct sockaddr_un *sa;
-	unsigned int sa_size;
+	unsigned int sa_len;
 	int sock;
 
 	if ((sock=socket(PF_UNIX,SOCK_STREAM,0))==-1) {
@@ -152,12 +183,12 @@ static int open_sock(const char *cache_dir)
 		exit(2);
 	}
 
-	sa_size = (offsetof(struct sockaddr_un, sun_path) + sizeof("/pdnsd.status") + strlen(cache_dir));
-	sa=(struct sockaddr_un *)alloca(sa_size);
+	sa_len = (offsetof(struct sockaddr_un, sun_path) + strlitlen("/pdnsd.status") + strlen(cache_dir));
+	sa=(struct sockaddr_un *)alloca(sa_len+1);
 	sa->sun_family=AF_UNIX;
 	stpcpy(stpcpy(sa->sun_path,cache_dir),"/pdnsd.status");
 
-	if (connect(sock,(struct sockaddr *)sa,sa_size)==-1) {
+	if (connect(sock,(struct sockaddr *)sa,sa_len)==-1) {
 		fprintf(stderr,"Error: could not open socket %s: %s\n",sa->sun_path,strerror(errno));
 		close(sock);
 		exit(2);
@@ -166,9 +197,9 @@ static int open_sock(const char *cache_dir)
 	return sock;
 }
 
-static void send_long(int fd,long cmd)
+static void send_long(int fd,uint32_t cmd)
 {
-	long nc=htonl(cmd);
+	uint32_t nc=htonl(cmd);
 
 	if (write(fd,&nc,sizeof(nc))!=sizeof(nc)) {
 		perror("Error: could not write long");
@@ -176,9 +207,9 @@ static void send_long(int fd,long cmd)
 	}
 }
 
-static void send_short(int fd,short cmd)
+static void send_short(int fd,uint16_t cmd)
 {
-	short nc=htons(cmd);
+	uint16_t nc=htons(cmd);
 
 	if (write(fd,&nc,sizeof(nc))!=sizeof(nc)) {
 		perror("Error: could not write short");
@@ -188,17 +219,17 @@ static void send_short(int fd,short cmd)
 
 static void send_string(int fd, const char *s)
 {
-	unsigned short len=strlen(s);
+	uint16_t len=(s?strlen(s):(~0));
 	send_short(fd,len);
-	if (write_all(fd,s,len)!=len) {
+	if (s && write_all(fd,s,len)!=len) {
 		perror("Error: could not write string");
 		exit(2);
 	}
 }
 
-static short read_short(int fd)
+static uint16_t read_short(int fd)
 {
-	short nc;
+	uint16_t nc;
 
 	if (read(fd,&nc,sizeof(nc))!=sizeof(nc)) {
 		perror("Error: could not read short");
@@ -226,21 +257,18 @@ static int copymsgtofile(int fd, FILE* out)
 
 static int match_cmd(const char *cmd, const cmd_s cmds[])
 {
-	int i=0;
-	while (cmds[i].name) {
+	int i;
+	for(i=0; cmds[i].name; ++i) {
 		if (strcmp(cmd,cmds[i].name)==0)
 			return cmds[i].val;
-		i++;
 	}
-	fprintf(stderr,"Command/option not recognized: %s\n",cmd);
-	fprintf(stderr,"Try 'pdnsd-ctl help' for available commands and options.\n");
-	exit(2);
+	return -1;
 }
 
 int main(int argc, char *argv[])
 {
-	char *cache_dir=NULL;
-	short rv=0;
+	char *cache_dir= CACHEDIR;
+	int rv=0;
 	{
 		int i;
 		char *arg;
@@ -266,76 +294,97 @@ int main(int argc, char *argv[])
 		argv += i;
 	}
 
-	if(!cache_dir) cache_dir= CACHEDIR;
-	  
 	if (argc<1) {
 		fprintf(stderr,"No command specified.\n");
 	print_try_pdnsd_ctl_help:
 		fprintf(stderr,"Try 'pdnsd-ctl help' for available commands and options.\n");
 		exit(2);
-	} else if (strcmp(argv[0],"help")==0) {
-		int i;
-		fputs(version_message,stdout);
-		for(i=0;i<num_help_messages;++i)
-			fputs(help_messages[i],stdout);
-		exit(0);
-	} else if (strcmp(argv[0],"version")==0) {
-		fputs(version_message,stdout);
-		exit(0);
-	} else if (strcmp(argv[0],"list-rrtypes")==0) {
-		int i;
-		printf("Available RR types for the neg command:\n");
-		for (i=0;i<T_NUM;i++)
-			printf("%s\n",rr_info[i].name);
-		exit(0);
 	} else {
-		int pf,acnt=0;
-		short cmd;
+		int pf,acnt=0,cmd;
 
 		cmd=match_cmd(argv[0],top_cmds);
-		pf=open_sock(cache_dir);
-		send_short(pf,cmd);
+		if(cmd==-1) {
+			fprintf(stderr,"Command not recognized: %s\n",argv[0]);
+			goto print_try_pdnsd_ctl_help;
+		}
 		switch (cmd) {
+		case CMD_HELP: {
+			int i;
+			fputs(version_message,stdout);
+			for(i=0;i<num_help_messages;++i)
+				fputs(help_messages[i],stdout);
+		}
+			break;
+
+		case CMD_VERSION:
+			fputs(version_message,stdout);
+			break;
+
+		case CMD_LIST_RRTYPES: {
+			int i;
+			if (argc!=1)
+				goto wrong_args;
+			printf("Available RR types for the neg command:\n");
+			for (i=0;i<T_NUM;i++)
+				printf("%s\n",rr_info[i].name);
+		}
+			break;
+
 		case CTL_STATS:
 			if (argc!=1)
 				goto wrong_args;
-			if(copymsgtofile(pf,stdout)<0) {
-				perror("Error while reading from socket");
-				exit(2);
-			}
-			break;
-		case CTL_SERVER:
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
+			goto copy_pf;
+
+		case CTL_SERVER: {
+			int server_cmd;
 			if (argc<3 || argc>4)
 				goto wrong_args;
+			acnt=2;
+			server_cmd=match_cmd(argv[2],server_cmds);
+			if(server_cmd==-1) goto bad_arg;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
 			send_string(pf,argv[1]);
-			send_short(pf,match_cmd(argv[2],server_cmds));
-			if(argc<4)
-				send_short(pf,~0);
-			else
-				send_string(pf,argv[3]);
+			send_short(pf,server_cmd);
+			send_string(pf,argc<4?NULL:argv[3]);
+		}
 			goto read_retval;
-		case CTL_RECORD:	
+
+		case CTL_RECORD: {
+			int record_cmd;
 			if (argc!=3)
 				goto wrong_args;
-			send_short(pf,match_cmd(argv[2],record_cmds));
+			acnt=2;
+			record_cmd=match_cmd(argv[2],record_cmds);
+			if(record_cmd==-1) goto bad_arg;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
+			send_short(pf,record_cmd);
 			send_string(pf,argv[1]);
+		}
 			goto read_retval;
+
 		case CTL_SOURCE: {
 			long ttl;
-			short servaliases,flags;
+			int servaliases,flags;
 			if (argc<3 || argc>6)
 				goto wrong_args;
 			ttl=900;
 			flags=DF_LOCAL;
 			acnt=3;
 			if (argc==6 || (argc>=4 && isdigit(argv[3][0]))) {
-				if (sscanf(argv[3],"%li",&ttl)!=1)
+				char *endptr;
+				ttl=strtol(argv[3],&endptr,0);
+				if (*endptr)
 					goto bad_arg;
 				acnt++;
 			}
 			servaliases=0;
 			if (acnt<argc && (argc==6 || strcmp(argv[acnt], "noauth"))) {
 				servaliases=match_cmd(argv[acnt],onoff_cmds);
+				if(servaliases==-1) goto bad_arg;
 				acnt++;
 			}
 			if (acnt<argc) {
@@ -344,6 +393,8 @@ int main(int argc, char *argv[])
 				else
 					goto bad_arg;
 			}
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
 			send_string(pf,argv[1]);
 			send_string(pf,argv[2]);
 			send_long(pf,ttl);
@@ -351,20 +402,34 @@ int main(int argc, char *argv[])
 			send_short(pf,flags);
 		}
 			goto read_retval;
+
 		case CTL_ADD: {
 			long ttl;
-			short tp,flags;
-			if (argc<2 ||
-			    argc<(acnt=(((tp=match_cmd(argv[1],rectype_cmds))==T_MX)?5:4)) ||
-			    argc>acnt+2)
-			{
+			int tp,flags,pref;
+			if (argc<2) goto wrong_args;
+			acnt=1;
+			tp=match_cmd(argv[1],rectype_cmds);
+			if(tp==-1) goto bad_arg;
+			acnt=((tp==T_MX)?5:4);
+			if (argc<acnt || argc>acnt+2)
 				goto wrong_args;
-			}
 
 			ttl=900;
 			flags=DF_LOCAL;
+			pref=0;
+			if(tp==T_MX) {
+				char *endptr;
+				pref=strtol(argv[4],&endptr,0);
+				if (*endptr) {
+					acnt=4;
+					goto bad_arg;
+				}
+			}
+
 			if (acnt<argc && strcmp(argv[acnt],"noauth")) {
-				if (sscanf(argv[acnt],"%li",&ttl)!=1)
+				char *endptr;
+				ttl=strtol(argv[acnt],&endptr,0);
+				if (*endptr)
 					goto bad_arg;
 				acnt++;
 			}
@@ -374,6 +439,8 @@ int main(int argc, char *argv[])
 			}
 			if (acnt<argc)
 				goto bad_arg;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
 			send_short(pf,tp);
 			send_string(pf,argv[3]);
 			send_long(pf,ttl);
@@ -415,15 +482,8 @@ int main(int argc, char *argv[])
 				send_string(pf,argv[2]);
 				break;
 			case T_MX:
-				{
-					short pref;
-					if (sscanf(argv[4], "%hd", &pref)!=1) {
-						fprintf(stderr,"Bad preference number for 'add mx' command: %s\n",argv[4]);
-						exit(2);
-					}
-					send_short(pf,pref);
-					send_string(pf,argv[2]);
-				}
+				send_short(pf,pref);
+				send_string(pf,argv[2]);
 				break;
 			}
 		}
@@ -431,7 +491,7 @@ int main(int argc, char *argv[])
 
 		case CTL_NEG: {
 			long ttl;
-			short tp;
+			int tp;
 
 			if (argc<2 || argc>4)
 				goto wrong_args;
@@ -440,33 +500,75 @@ int main(int argc, char *argv[])
 			acnt=2;
 			if (argc==3) {
 				if (isdigit(argv[2][0])) {
-					if (sscanf(argv[2],"%li",&ttl)!=1)
+					char *endptr;
+					ttl=strtol(argv[2],&endptr,0);
+					if (*endptr)
 						goto bad_arg;
 				} else if ((tp=rr_tp_byname(argv[2]))==-1) {
 					goto bad_type;
 				}
 			} else if (argc==4) {
+				char *endptr;
 				if ((tp=rr_tp_byname(argv[2]))==-1)
 					goto bad_type;
-				if (sscanf(argv[3],"%li",&ttl)!=1) {
+				ttl=strtol(argv[3],&endptr,0);
+				if (*endptr) {
 					acnt=3;
 					goto bad_arg;
 				}
 			}
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
 			send_string(pf,argv[1]);
 			send_short(pf,tp);
 			send_long(pf,ttl);
 		}
-			/* fall through */
+			goto read_retval;
+
+		case CTL_CONFIG:
+			if (argc>2)
+				goto wrong_args;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
+			send_string(pf,argc<2?NULL:argv[1]);
+			goto read_retval;
+
+		case CTL_EMPTY:
+			if (argc!=1)
+				goto wrong_args;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
+			goto read_retval;
+
+		case CTL_DUMP:
+			if (argc>2)
+				goto wrong_args;
+			pf=open_sock(cache_dir);
+			send_short(pf,cmd);
+			send_string(pf,argc<2?NULL:argv[1]);
+			if((rv=read_short(pf)))
+				goto retval_failed;
+		copy_pf:
+			if(copymsgtofile(pf,stdout)<0) {
+				perror("Error while reading from socket");
+				exit(2);
+			}
+			goto close_pf;
 
 		read_retval:
 			if((rv=read_short(pf))) {
+			retval_failed:
 				fprintf(stderr,"Failed: ");
 				if(copymsgtofile(pf,stderr)<0)
 					fprintf(stderr,"(could not read error message from socket: %s)",strerror(errno));
 
 				fputc('\n',stderr);
 			}
+		close_pf:
+			if(close(pf)==-1)
+				perror("Couldn't close socket");
+			else if (rv==0 && verbose)
+				printf("Succeeded\n");
 			break;
 		wrong_args:
 			fprintf(stderr,"Wrong number of arguments for '%s' command.\n",argv[0]);
@@ -484,10 +586,6 @@ int main(int argc, char *argv[])
 				argv[0],argv[acnt]);
 			exit(2);
 		}
-		if(close(pf)==-1)
-			perror("Couldn't close socket");
-		else if (rv==0 && verbose)
-			printf("Succeeded\n");
 	}
 
 	return rv;
