@@ -2,6 +2,8 @@
 #include <pthread.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include "../helpers.h"
+#include "../conff.h"
 #include "../netdev.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
@@ -21,6 +23,7 @@ pthread_t main_thread;
 #if DEBUG>0
 FILE *dbg_file;
 #endif
+globparm_t global;
 
 
 int main(int argc, char *argv[]) 
