@@ -3,6 +3,7 @@
 #include <pthread.h>
 #include <string.h>
 #include "../helpers.h"
+#include "../conff.h"
 
 #if !defined(lint) && !defined(NO_RCSIDS)
 static char rcsid[]="$Id: random.c,v 1.1 2000/07/20 20:03:25 thomas Exp $";
@@ -21,6 +22,7 @@ pthread_t main_thread;
 #if DEBUG>0
 FILE *dbg_file;
 #endif
+globparm_t global;
 
 
 int main(void) 
