@@ -1,7 +1,7 @@
 /* thread.c - Threading helpers
-   Copyright (C) 2000 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2002, 2003.
+   Copyright (C) 2000 Thomas Moestl
+   Copyright (C) 2002, 2003 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -40,13 +40,15 @@ static char rcsid[]="$Id: thread.c,v 1.5 2001/05/19 14:57:30 tmm Exp $";
 
 volatile short int waiting=0; /* Has the main thread already done sigwait() ? */
 pthread_attr_t attr_detached;
+#if DEBUG>0
 pthread_key_t thrid_key;
+#endif
 
 /* This is a handler for signals to the threads. We just hand the sigs on to the main thread.
  * Note that this may result in blocked locks. We have no means to open the locks here, because in LinuxThreads
  * the mutex functions are not async-signal safe. So, locks may still be active. We account for this by using
  * softlocks (see below) in any functions called after sigwait from main(). */
-#if TARGET==TARGET_LINUX && !defined(THREADLIB_NPTL)
+#if (TARGET==TARGET_LINUX) && !defined(THREADLIB_NPTL)
 void thread_sig(int sig)
 {
 	if (sig==SIGTSTP || sig==SIGTTOU || sig==SIGTTIN) {
@@ -66,9 +68,11 @@ void thread_sig(int sig)
 }
 #endif
 
-/* void usleep_r(unsigned long usec)
+/* This is now defined as an inline function in thread.h */
+#if 0
+void usleep_r(unsigned long usec)
 {
-#if (TARGET==TARGET_LINUX || TARGET==TARGET_BSD) && defined(HAVE_USLEEP)
+#if ((TARGET==TARGET_LINUX) || (TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)) && defined(HAVE_USLEEP)
 	usleep(usec);
 #else
 	struct timeval tv;
@@ -77,5 +81,6 @@ void thread_sig(int sig)
 	tv.tv_usec=usec%1000000;
 	select(0, NULL, NULL, NULL, tv);
 #endif
-} */
+}
+#endif
 
