@@ -1,7 +1,7 @@
 /* thread.h - Threading helpers
-   Copyright (C) 2000 Thomas Moestl
 
-   With modifications by Paul Rombouts, 2002, 2003.
+   Copyright (C) 2000 Thomas Moestl
+   Copyright (C) 2002, 2003 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -33,7 +33,7 @@ Boston, MA 02111-1307, USA.  */
 extern sigset_t sigs_msk;
 /* --- */
 
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX) && !defined(THREADLIB_NPTL)
 void thread_sig(int sig);
 #endif
 
@@ -52,7 +52,7 @@ void thread_sig(int sig);
    The signals are blocked in main() before any threads are created,
    and we simply never unblock them except by calling sigwait() in main(). */
 
-#if TARGET==TARGET_LINUX
+#if (TARGET==TARGET_LINUX)
 # ifdef THREADLIB_NPTL
 # define THREAD_SIGINIT
 # else
@@ -82,8 +82,10 @@ void thread_sig(int sig);
 			   }
 #  endif
 # endif
-#else
+#elif (TARGET==TARGET_BSD) || (TARGET==TARGET_CYGWIN)
 #define THREAD_SIGINIT pthread_sigmask(SIG_BLOCK,&sigs_msk,NULL)
+#else
+# error Unsupported platform!
 #endif
 
 
@@ -113,7 +115,9 @@ inline static int sleep_r (unsigned int seconds)
 /* Used for creating detached threads */
 extern pthread_attr_t attr_detached;
 
+#if DEBUG>0
 /* Key for storing private thread ID's */
 extern pthread_key_t thrid_key;
+#endif
 
 #endif
